## Meghan Sullivan
## Chong Xing
## 2017-06-01

## SEM Examples for Using Likert Items from hbsc Data
## Section-1: One-Factor Categorical CFA (with Likert Items)
## Section-2: Three-Factor Categorical CFA (Likert)
## Section-3: Two-Factor Categorical Structural Model (Likert)
## Section-4: Indirect-Effect (Mediation) Structural Model (Likert)
## Section-5: One-Factor Two-Group Measurement Invariance Testing (Likert)

wdir <- "workingdata"
ddir <- "../../data"
odir <- "output"

library(kutils)
library(foreign)
library(lavaan)

pdf.options(onefile=FALSE, family="Times", paper="special", height=4,
            width=6, pointsize=10)

## Read in Data
hbsc <- readRDS(file = file.path(ddir, "hbsc-subset.rds"))
summary(hbsc)



####---------------------------------------------####
#### Section-1: One-Factor CFA with Likert Items ####
####---------------------------------------------####

#### Three Versions of One-Factor CFA for Depression Items
#### A Six-Item Five-Point Likert Scale
#### ("Depress1_f" Coded as Factor Variable)
#### ("Deqpress1_i" Coded as Integer Varable)
#### peek() function in kutils package gives
#### a quick inspection of the variables  

class(hbsc$Depress1_f)
class(hbsc$Depress1_i)

if(interactive()) peek(hbsc[ , c("Depress1_f", "Depress1_i", "Depress2_f", "Depress2_i",
               "Depress3_f", "Depress3_i", "Depress4_f", "Depress4_i",
               "Depress5_f", "Depress5_i", "Depress6_f", "Depress6_i")])

## Version 1 - ML Estimation with Factor Variables
## lavaan Gives an Error Message about Unordered Factor(s)
## lavaan will still Provide Parameter Estimates
## But NAs will be Assigned to Loglikelihood, AIC, and BIC

model.depress.factor <- '
depress =~ NA*Depress1_f + Depress2_f + Depress3_f + Depress4_f
           + Depress5_f + Depress6_f
depress ~~ 1*depress
'

fit.depress.ML.factor <- cfa(model = model.depress.factor, data = hbsc,
                             mimic = "Mplus", estimator = "ML")

summary(fit.depress.ML.factor, fit.measures = TRUE, standardized = TRUE)

## Version 2 - ML Estimation with Integer Variables
## The Error Message about Unordered Factor Is Gone
## Estimates on Loglikelihood, AIC, and BIC Are Provided
## All the Parameter Estimates Are Identical to the Version 1

model.depress.integer <- '
depress =~ NA*Depress1_i + Depress2_i + Depress3_i + Depress4_i
           + Depress5_i + Depress6_i
depress ~~ 1*depress
'

fit.depress.ML.integer <- cfa(model = model.depress.integer, data = hbsc,
                              mimic = "Mplus", estimator = "ML")

summary(fit.depress.ML.integer, fit.measures = TRUE, standardized = TRUE)

## Categorical Treatment of the Items (WLSMV)
## peek() Function Showed Items 1, 3, 4, 5, 6
## Were Not Normally Distributed (They Are Declared as "ordered") 

fit.depress.WLSMV.factor <-
    cfa(model = model.depress.factor, data = hbsc,
        mimic = "Mplus", estimator = "WLSMV",
        ordered = c("Depress1_f", "Depress2_f", "Depress3_f",
                    "Depress4_f", "Depress5_f", "Depress6_f"))

summary(fit.depress.WLSMV.factor, fit.measures = TRUE, standardized = TRUE)



#### One-Factor CFA for Bullying Others Items (e.g., "BullyOth1")
#### A Nine-Item Five-Point Likert Scale
#### ("BullyOther1_f" Coded as Factor Variable)
#### ("BullyOther1_i" Coded as Integer Varable)

if(interactive()) peek(hbsc[ , c("BullyOth1_f", "BullyOth1_i", "BullyOth2_f", "BullyOth2_i",
               "BullyOth3_f", "BullyOth3_i", "BullyOth4_f", "BullyOth4_i",
               "BullyOth5_f", "BullyOth5_i", "BullyOth6_f", "BullyOth6_i",
               "BullyOth7_f", "BullyOth7_i", "BullyOth8_f", "BullyOth8_i",
               "BullyOth9_f", "BullyOth9_i")])

## Continuous Treatment of the Items (ML)
model.bullyOther.integer <- '
bullyOther =~ NA*BullyOth1_i + BullyOth2_i + BullyOth3_i
              + BullyOth4_i  + BullyOth5_i + BullyOth6_i
              + BullyOth7_i + BullyOth8_i  + BullyOth9_i
bullyOther ~~ 1*bullyOther
'

fit.bullyOther.ML.integer <-
    cfa(model = model.bullyOther.integer, data = hbsc,
        mimic = "Mplus", estimator = "ML")

summary(fit.bullyOther.ML.integer, fit.measures = TRUE, standardized = TRUE)

## Categorical Treatment of the Items (WLSMV)
model.bullyOther.factor <- '
bullyOther =~ NA*BullyOth1_f + BullyOth2_f + BullyOth3_f
              + BullyOth4_f + BullyOth5_f + BullyOth6_f
              + BullyOth7_f + BullyOth8_f + BullyOth9_f
bullyOther ~~ 1*bullyOther
'

fit.bullyOther.WLSMV.factor <-
    cfa(model = model.bullyOther.factor, data = hbsc,
        mimic = "Mplus", estimator = "WLSMV",
        ordered = c("BullyOth1_f", "BullyOth2_f",
                    "BullyOth3_f", "BullyOth4_f",
                    "BullyOth5_f", "BullyOth6_f",
                    "BullyOth7_f", "BullyOth8_f",
                    "BullyOth9_f"))

summary(fit.bullyOther.WLSMV.factor, fit.measures = TRUE, standardized = TRUE)



#### One-Factor CFA for Got Bullied Items
#### A Nine-Item Five-Point Likert Scale
#### ("GotBully1_f" Coded as Factor Variable)
#### ("GotBully1_i" Coded as Integer Varable)

if(interactive()) peek(hbsc[ , c("GotBully1_f", "GotBully1_i", "GotBully2_f", "GotBully2_i",
               "GotBully3_f", "GotBully3_i", "GotBully4_f", "GotBully4_i",
               "GotBully5_f", "GotBully5_i", "GotBully6_f", "GotBully6_i",
               "GotBully7_f", "GotBully7_i", "GotBully8_f", "GotBully8_i",
               "GotBully9_f", "GotBully9_i")])

## Continuous Treatment of the Items (ML)
model.gotBully.integer <- '
gotBully =~ NA*GotBully1_i + GotBully2_i + GotBully3_i
            + GotBully4_i  + GotBully5_i + GotBully6_i 
            + GotBully7_i + GotBully8_i + GotBully9_i
gotBully ~~ 1*gotBully
'
 
fit.gotBully.ML.integer <- cfa(model = model.gotBully.integer, data = hbsc,
                               mimic = "Mplus", estimator = "ML")

summary(fit.gotBully.ML.integer, fit.measures = TRUE, standardized = TRUE)

## Categorical Treatment of the Items (WLSMV)
model.gotBully.factor <- '
gotBully =~ NA*GotBully1_f + GotBully2_f + GotBully3_f
            + GotBully4_f  + GotBully5_f + GotBully6_f 
            + GotBully7_f + GotBully8_f  + GotBully9_f
gotBully ~~ 1*gotBully
'

fit.gotBully.WLSMV.factor <-
    cfa(model = model.gotBully.factor, data = hbsc,
        mimic = "Mplus", estimator = "WLSMV",
        ordered = c("GotBully1_f", "GotBully2_f",
                    "GotBully3_f", "GotBully4_f",
                    "GotBully5_f", "GotBully6_f",
                    "GotBully7_f", "GotBully8_f",
                    "GotBully9_f"))

summary(fit.gotBully.WLSMV.factor, fit.measures = TRUE, standardized = TRUE)



#### One-Factor CFA for Body Feelings items
#### A Five-Item Five-Point Likert Scale
#### ("BodyFeelings1_f" Coded as Factor Variable)
#### ("BodyFeelings1_i" Coded as Integer Varable)

if(interactive()) peek(hbsc[ , c("BodyFeelings1_f", "BodyFeelings1_i",
               "BodyFeelings2_f", "BodyFeelings2_i",
               "BodyFeelings3_f", "BodyFeelings3_i",
               "BodyFeelings4_f", "BodyFeelings4_i",
               "BodyFeelings5_f", "BodyFeelings5_i")])

model.bodyFeelings.integer <- '
bodyFeelings =~ NA*BodyFeelings1_i + BodyFeelings2_i
                + BodyFeelings3_i + BodyFeelings4_i
                + BodyFeelings5_i
bodyFeelings ~~ 1*bodyFeelings'

## Continuous Treatment of the Items (ML)
fit.bodyFeelings.ML.integer <-
    cfa(model = model.bodyFeelings.integer, data = hbsc,
        mimic = "Mplus", estimator = "ML")

summary(fit.bodyFeelings.ML.integer, fit.measures = TRUE, standardized = TRUE)

## Categorical Treatment of the Items (WLSMV)
model.bodyFeelings.factor <- '
bodyFeelings =~ NA*BodyFeelings1_f + BodyFeelings2_f
                + BodyFeelings3_f + BodyFeelings4_f
                + BodyFeelings5_f
bodyFeelings ~~ 1*bodyFeelings
'

fit.bodyFeelings.WLSMV.factor <-
    cfa(model = model.bodyFeelings.factor, data = hbsc,
        mimic = "Mplus", estimator = "WLSMV",
        ordered = c("BodyFeelings1_f", "BodyFeelings2_f",
                    "BodyFeelings3_f", "BodyFeelings4_f",
                    "BodyFeelings5_f"))

summary(fit.bodyFeelings.WLSMV.factor, fit.measures = TRUE,
        standardized = TRUE)



#### One-Factor CFA for Physical Health Items
#### An Eight-Item Five-Point Likert Scale
#### ("PhysHealth1_f" Coded as Factor Variable)
#### ("PhysHealth1_i" Coded as Integer Varable)

if(interactive()) peek(hbsc[ , c("PhysHealth1_f", "PhysHealth1_i", "PhysHealth2_f", "PhysHealth2_i",
               "PhysHealth3_f", "PhysHealth3_i", "PhysHealth4_f", "PhysHealth4_i",
               "PhysHealth5_f", "PhysHealth5_i", "PhysHealth6_f", "PhysHealth6_i",
               "PhysHealth7_f", "PhysHealth7_i", "PhysHealth8_f", "PhysHealth8_i")])

## Continuous Treatment of the Items (ML)
model.physHealth.integer <- '
physHealth =~ NA*PhysHealth1_i + PhysHealth2_i
               + PhysHealth3_i + PhysHealth4_i
               + PhysHealth5_i + PhysHealth6_i
               + PhysHealth7_i + PhysHealth8_i
physHealth ~~ 1*physHealth'

fit.physHealth.ML.integer <-
    cfa(model = model.physHealth.integer, data = hbsc,
        mimic = "Mplus", estimator = "ML")

summary(fit.physHealth.ML.integer, fit.measures = TRUE,
        standardized = TRUE)

## Categorical Treatment of the Items (WLSMV)
model.physHealth.factor <- '
physHealth =~ NA*PhysHealth1_f + PhysHealth2_f
              + PhysHealth3_f + PhysHealth4_f
              + PhysHealth5_f + PhysHealth6_f
              + PhysHealth7_f + PhysHealth8_f
physHealth ~~ 1*physHealth'

fit.physHealth.WLSMV.factor <-
    cfa(model = model.physHealth.factor, data = hbsc, mimic = "Mplus",
        estimator = "WLSMV",
        ordered = c("PhysHealth1_f", "PhysHealth2_f",
                    "PhysHealth3_f", "PhysHealth4_f",
                    "PhysHealth5_f", "PhysHealth6_f",
                    "PhysHealth7_f", "PhysHealth8_f"))

summary(fit.physHealth.WLSMV.factor, fit.measures = TRUE,
        standardized = TRUE)



#### One-Factor CFA for Alcohol-Use Items
#### A Five-Item Five-Point Likert Scale
#### ("Alc1_f" Coded as Factor Variable)
#### ("Alc1_i" Coded as Integer Varable)

if(interactive()) peek(hbsc[ , c("Alc1_f", "Alc1_i", "Alc2_f", "Alc2_i",
               "Alc3_f", "Alc3_i", "Alc4_f", "Alc4_i",
               "Alc5_f", "Alc5_i")])

## Continuous Treatment of the Items (ML)
model.alcohol.integer <- '
alcohol =~ NA*Alc1_i + Alc2_i + Alc3_i + Alc4_i + Alc5_i
alcohol ~~ 1*alcohol
'

fit.alcohol.ML.integer <-
    cfa(model = model.alcohol.integer, data = hbsc,
        mimic = "Mplus", estimator = "ML")

summary(fit.alcohol.ML.integer, fit.measures = TRUE,
        standardized = TRUE)

## Categorical Treatment of the Items (WLSMV)
model.alcohol.factor <- '
alcohol =~ NA*Alc1_f + Alc2_f + Alc3_f + Alc4_f + Alc5_f
alcohol ~~ 1*alcohol
'

fit.alcohol.WLSMV.factor <-
    cfa(model = model.alcohol.factor, data = hbsc, mimic = "Mplus",
        estimator = "WLSMV",
        ordered = c("Alc1_f", "Alc2_f", "Alc3_f", "Alc4_f", "Alc5_f"))

summary(fit.alcohol.WLSMV.factor, fit.measures = TRUE,
        standardized = TRUE)



####-------------------------------------------------####
#### Section-2: A Three-Factor CFA with Likert Items ####
#### Factors: Got Bullied, Depression, Alchol Use    ####
####-------------------------------------------------####

## The Continous Treatment (ML) of the Items Coded as Interger Variables
model.CFA.integer <- '
gotBully =~ NA*GotBully1_i + GotBully2_i + GotBully3_i
            + GotBully4_i + GotBully5_i + GotBully6_i
            + GotBully7_i + GotBully8_i + GotBully9_i
gotBully ~~ 1*gotBully

depress =~ NA*Depress1_i + Depress2_i + Depress3_i
           + Depress4_i + Depress5_i + Depress6_i
depress ~~ 1*depress
alcohol =~ NA*Alc1_i + Alc2_i + Alc3_i + Alc4_i + Alc5_i
alcohol ~~ 1*alcohol
'

fit.CFA.ML.integer <- cfa(model = model.CFA.integer, data = hbsc,
                          mimic = "Mplus", estimator = "ML")

summary(fit.CFA.ML.integer, fit.measures = TRUE, standardized = TRUE)

## The Categorical Treatment (WLSMV) of the Items Coded as Factor Variables
model.CFA.factor <- '
gotBully =~ NA*GotBully1_f + GotBully2_f + GotBully3_f
            + GotBully4_f + GotBully5_f + GotBully6_f
            + GotBully7_f + GotBully8_f + GotBully9_f
gotBully ~~ 1*gotBully

depress =~ NA*Depress1_f + Depress2_f + Depress3_f
           + Depress4_f + Depress5_f + Depress6_f
depress ~~ 1*depress

alcohol =~ NA*Alc1_f + Alc2_f + Alc3_f + Alc4_f + Alc5_f
alcohol ~~ 1*alcohol
'

fit.CFA.WLSMV.factor <-
    cfa(model = model.CFA.factor, data = hbsc, mimic = "Mplus",
        estimator = "WLSMV",
        ordered = c("GotBully1_f", "GotBully2_f","GotBully3_f",
                    "GotBully4_f", "GotBully5_f", "GotBully6_f",
                    "GotBully7_f", "GotBully8_f", "GotBully9_f",
                    "Depress1_f", "Depress2_f", "Depress3_f",
                    "Depress4_f", "Depress5_f", "Depress6_f",
                    "Alc1_f", "Alc2_f", "Alc3_f", "Alc4_f", "Alc5_f"))

summary(fit.CFA.WLSMV.factor, fit.measures = TRUE, standardized = TRUE)



####------------------------------------------------------------####
#### Section-3: A Two-Factor Structural Model with Likert Items ####
#### Got Bullied Predicts Alcohol Use                           ####
####------------------------------------------------------------####

## The Continuous Treatment (ML) of the Structural Model
## The Variables for this Model Are Coded as Integers

model.struc.ML <- '
## the measurement model
gotBully =~ NA*GotBully1_i + GotBully2_i + GotBully3_i
            + GotBully4_i + GotBully5_i + GotBully6_i
            + GotBully7_i + GotBully8_i + GotBully9_i
gotBully ~~ 1*gotBully

alcohol =~ NA*Alc1_i + Alc2_i + Alc3_i + Alc4_i + Alc5_i
alcohol ~~ 1*alcohol

# regress 
alcohol ~ gotBully
'

fit.struc.ML <- sem(model = model.struc.ML, data = hbsc,
                    mimic = "Mplus", estimator = "ML")

summary(fit.struc.ML, fit.measures = TRUE, standardized = TRUE)

## The Categorical Treatment (WLSMV) of the Structural Model
## The Variables for this Model Are Coded as Factors

model.struc.WLSMV <- '
## the measurement model
gotBully =~ NA*GotBully1_f + GotBully2_f + GotBully3_f + GotBully4_f
            + GotBully5_f + GotBully6_f  + GotBully7_f + GotBully8_f
            + GotBully9_f
gotBully ~~ 1*gotBully

alcohol =~ NA*Alc1_f + Alc2_f + Alc3_f + Alc4_f + Alc5_f
alcohol ~~ 1*alcohol

## the structural model
alcohol ~ gotBully
'

fit.struc.WLSMV <-
    sem(model = model.struc.WLSMV, data = hbsc, mimic = "Mplus",
        estimator = "WLSMV", ordered = c("GotBully1_f", "GotBully2_f",
                                         "GotBully3_f", "GotBully4_f",
                                         "GotBully5_f", "GotBully6_f",
                                         "GotBully7_f", "GotBully8_f",
                                         "GotBully9_f",
                                         "Alc1_f", "Alc2_f", "Alc3_f",
                                         "Alc4_f", "Alc5_f"))

summary(fit.struc.WLSMV, fit.measures = TRUE, standardized = TRUE)



####--------------------------------------------####
#### Section-4: A Three-Factor Structural Model ####
#### Depression as a potential Mediator between #### 
#### Got Bullied and Alcohol Use                ####
####--------------------------------------------####

## The Continuous Treatment (ML) of the Indirect-Effect (Mediation) Model
## The Variables for this Model Are Coded as Integers

model.indirect.con <- '
## the measurement model

gotBully =~ NA*GotBully1_i + GotBully2_i + GotBully3_i
             + GotBully4_i + GotBully5_i + GotBully6_i 
             + GotBully7_i + GotBully8_i + GotBully9_i
gotBully ~~ 1*gotBully

depress =~ NA*Depress1_i + Depress2_i + Depress3_i
         + Depress4_i + Depress5_i + Depress6_i
depress ~~ 1*depress

alcohol =~ NA*Alc1_i + Alc2_i + Alc3_i + Alc4_i + Alc5_i
alcohol ~~ 1*alcohol

## the structural model
# direct effect (the c path)
alcohol ~ c*gotBully
# mediator paths (the a and b path)
depress ~ a*gotBully # the a path - IV predicting ME
alcohol ~ b*depress  # the b path - ME predicting DV

# indirect effect (a*b)
# := operator defines new parameters
ab := a*b

# total effect
total := c + (a*b)
'

## verbose = FALSE will not display the bootsrap iterations on screen
fit.indirect.ML <- sem(model = model.indirect.con, data = hbsc,
                       mimic = "Mplus", estimator = "ML",
                       se = "bootstrap", verbose = TRUE, bootstrap = 10)

summary(fit.indirect.ML, fit.measures = TRUE, standardized = TRUE)



## The Categorical Treatment (DWLS) of the Indirect-Effect (Mediation) Model
## The Variables for this Model Are Coded as Factors

model.indirect.cat <- '
## the measurement model

gotBully =~ NA*GotBully1_f + GotBully2_f + GotBully3_f
             + GotBully4_f + GotBully5_f + GotBully6_f 
             + GotBully7_f + GotBully8_f + GotBully9_f
gotBully ~~ 1*gotBully

depress =~ NA*Depress1_f + Depress2_f + Depress3_f
            + Depress4_f + Depress5_f + Depress6_f
depress ~~ 1*depress

alcohol =~ NA*Alc1_f + Alc2_f + Alc3_f + Alc4_f + Alc5_f
alcohol ~~ 1*alcohol

## the structural model

## direct effect (the c path)
alcohol ~ c*gotBully

## mediator paths (the a and b path)
depress ~ a*gotBully # the a path - IV predicting ME
alcohol ~ b*depress  # the b path - ME predicting DV

## indirect effect (a*b)
## := operator defines new parameters
ab := a*b

## total effect
total := c + (a*b)
'

fit.indirect.DWLS <-
    sem(model = model.indirect.cat, data = hbsc, mimic = "Mplus",
        estimator = "DWLS", se = "bootstrap", verbose = TRUE,
        bootstrap = 10,
        ordered = c("GotBully1_f", "GotBully2_f", "GotBully3_f",
                    "GotBully4_f", "GotBully5_f", "GotBully6_f",
                    "GotBully7_f", "GotBully8_f", "GotBully9_f",
                    "Depress1_f", "Depress2_f", "Depress3_f",
                    "Depress4_f", "Depress5_f", "Depress6_f",
                    "Alc1_f", "Alc2_f", "Alc3_f", "Alc4_f", "Alc5_f"))

summary(fit.indirect.DWLS, fit.measures = TRUE, standardized = TRUE)



####-------------------------------------------------------------####
#### Section-5: One- Factort Two-Group Measurement Invariance    ####
#### A One-Factor (Depression) CFA Model between Grade 6 and 7   ####
####-------------------------------------------------------------####

## Creating a Dataset only Containing Observations from Grade 6 and 7
dat.inva <- hbsc[hbsc$Grade %in% c("Grade_6", "Grade_7"), ]
dim(dat.inva)

## The Continous Treatment (ML) of the Items Coded as Interger Variables

## 1. Configural Invariance Test (ML)
model.config.ML <- '
depress =~ Depress1_i + Depress2_i + Depress3_i + Depress4_i
           + Depress5_i + Depress6_i
'

fit.config.ML <- cfa(model = model.config.ML, data = dat.inva,
                     mimic = "Mplus", estimator = "ML",
                     std.lv = TRUE, group = "Grade")

summary(fit.config.ML, fit.measures = TRUE, standardized = TRUE)

## 2. Metric (Weak) Invariance Test (ML)
model.metric.ML <-'
depress =~ Depress1_i + Depress2_i + Depress3_i + Depress4_i
           + Depress5_i + Depress6_i
depress ~~ c(1, NA)*depress'

fit.metric.ML <- cfa(model = model.config.ML, data = dat.inva,
                     mimic = "Mplus", estimator = "ML",
                     std.lv = TRUE, group = "Grade",
                     group.equal = "loadings")

summary(fit.metric.ML, fit.measures = TRUE, standardized = TRUE)

lavTestLRT(fit.config.ML, fit.metric.ML)

## 3. Scalar (Strong) Invariance Test (ML)
model.scalar.ML <- '
depress =~ Depress1_i + Depress2_i  + Depress3_i + Depress4_i
           + Depress5_i + Depress6_i
depress ~~ (1, NA)*depress
depress ~ c(0, NA)*1
'

fit.scalar.ML <- cfa(model = model.config.ML, data = dat.inva,
                     mimic = "Mplus", estimator = "ML",
                     std.lv = TRUE, group = "Grade",
                     group.equal = c("loadings", "intercepts"))

summary(fit.scalar.ML, fit.measures = TRUE, standardized = TRUE)

lavTestLRT(fit.metric.ML, fit.scalar.ML)

## The Categorical Treatment (WLSMV) of the Items Coded as Factor Variables
## 1. Configural Invariance Test (WLSMV)
model.config.WLSMV <- '
depress =~ Depress1_f + Depress2_f + Depress3_f
           + Depress4_f + Depress5_f + Depress6_f
'

fit.config.WLSMV <- cfa(model = model.config.WLSMV, data = dat.inva,
                        mimic = "Mplus", estimator = "WLSMV",
                        std.lv = TRUE, group = "Grade",
                        ordered = c("Depress1_f", "Depress2_f",
                                    "Depress3_f", "Depress4_f",
                                    "Depress5_f", "Depress6_f"))

summary(fit.config.WLSMV, fit.measures = TRUE, standardized = TRUE)

## 2. Metric (Weak) Invariance Test (WLSMV)
model.metric.WLSMV <- '
depress =~ Depress1_f + Depress2_f + Depress3_f
           + Depress4_f + Depress5_f + Depress6_f
depress ~~ c(1, NA)*depress'

fit.metric.WLSMV <- cfa(model = model.metric.WLSMV, data = dat.inva,
                        mimic = "Mplus", estimator = "WLSMV",
                        std.lv = TRUE, group = "Grade",
                        ordered = c("Depress1_f", "Depress2_f",
                                    "Depress3_f", "Depress4_f",
                                    "Depress5_f", "Depress6_f"),
                        group.equal = "loadings")

summary(fit.metric.WLSMV, fit.measures = TRUE, standardized = TRUE)

lavTestLRT(fit.config.WLSMV, fit.metric.WLSMV, method = "satorra.bentler.2010", A.method = "delta") 

## 3. Scalar (Strong) Invariance Test (WLSMV)
model.scalar.WLSMV <- '
depress =~ Depress1_f + Depress2_f + Depress3_f
           + Depress4_f + Depress5_f + Depress6_f
depress ~~ c(1, NA)*depress
depress ~ c(0, NA)*1
'

fit.scalar.WLSMV <- cfa(model = model.scalar.WLSMV, data = dat.inva,
                        mimic = "Mplus", estimator = "WLSMV",
                        std.lv = TRUE, group = "Grade",
                        ordered = c("Depress1_f", "Depress2_f",
                                    "Depress3_f", "Depress4_f",
                                    "Depress5_f", "Depress6_f"),
                        group.equal = c("loadings", "thresholds"))

summary(fit.scalar.WLSMV, fit.measures = TRUE, standardized = TRUE)

lavTestLRT(fit.metric.WLSMV, fit.scalar.WLSMV, method = "satorra.bentler.2010", A.method = "delta")

## 4. Metric and Scalar Combined Test (WLSMV)

lavTestLRT(fit.config.WLSMV, fit.scalar.WLSMV, method = "satorra.bentler.2010", A.method = "delta")
